<?php

/**
 * @file
 * Contains \Drupal\pc\EventSubscriber.
 */

namespace Drupal\pc;

use Drupal\Component\Utility\Timer;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Database\Connection;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\Core\Session\AccountInterface;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\HttpKernel\Event\FilterResponseEvent;
use Symfony\Component\HttpKernel\Event\GetResponseEvent;
use Symfony\Component\HttpKernel\KernelEvents;

/**
 * PHP Console event subscriber.
 */
class EventSubscriber implements EventSubscriberInterface {

  /**
   * Current logged in user.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $currentUser;

  /**
   * PHP Console settings.
   *
   * @var array
   */
  protected $settings;

  /**
   * Database logger.
   *
   * @var \Drupal\Core\Database\Log
   */
  protected $databaseLogger;

  /**
   * The current route match.
   *
   * @var \Drupal\Core\Routing\RouteMatchInterface
   */
  protected $routeMatch;

  /**
   * Debug info.
   *
   * @var array
   */
  protected $debugInfo;

  /**
   * Constructs event subscriber.
   *
   * @param \Drupal\Core\Session\AccountInterface $current_user
   *   Current logged in user.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   Configuration object factory.
   * @param \Drupal\Core\Database\Connection $db_connection
   *   Database connection.
   * @param \Drupal\Core\Routing\RouteMatchInterface $route_match
   *   The current route match.
   * @param \Drupal\pc\ConnectorFactory $connector_factory
   *   Connector factory.
   */
  public function __construct(AccountInterface $current_user, ConfigFactoryInterface $config_factory, Connection $db_connection, RouteMatchInterface $route_match, ConnectorFactory $connector_factory) {
    // Sometimes we need to build connector explicitly.
    $connector_factory->get();

    $this->currentUser = $current_user;
    $this->isAdmin = $this->currentUser->hasPermission('administer site configuration');
    $this->settings = $config_factory->get('pc.settings');
    $this->databaseLogger = $db_connection->getLogger();
    $this->routeMatch = $route_match;
    $this->debugInfo = $this->settings->get('debug_info');
  }

  /**
   * Kernel request event handler.
   *
   * @param \Symfony\Component\HttpKernel\Event\GetResponseEvent $event
   *   Response event.
   */
  public function onKernelRequest(GetResponseEvent $event) {

    // @TODO: Move this to middleware.
    Timer::start('pc_page');

    $this->debugInfo['server'] && pc($_SERVER, 'Server');
    $this->debugInfo['session'] && pc($_SESSION, 'Session');
    $this->debugInfo['cookie'] && pc($_COOKIE, 'Cookie');
    $this->debugInfo['post'] && pc($_POST, 'Post');
    $this->debugInfo['get'] && pc($_GET, 'Get');
    $this->debugInfo['logged_user'] && pc($this->currentUser, 'Logged user');

    if ($this->debugInfo['route']) {
      $route_info = [
        'name' => $this->routeMatch->getRouteName(),
        'object' => $this->routeMatch->getRouteObject(),
        'parameters' => $this->routeMatch->getParameters(),
        'row_parameters' => $this->routeMatch->getRawParameters(),
      ];
      pc($route_info, 'Route');
    }

  }

  /**
   * Kernel response event handler.
   *
   * @param \Symfony\Component\HttpKernel\Event\FilterResponseEvent $event
   *   Response event.
   */
  public function onKernelResponse(FilterResponseEvent $event) {

    $this->debugInfo['execution_time'] && pc(Timer::read('pc_page') . ' ms', 'Execution time');
    $this->debugInfo['memory_usage'] && pc(round(memory_get_usage() / 1024 / 1024, 2), 'Memory usage');
    $this->debugInfo['peak_memory_usage'] && pc(round(memory_get_peak_usage(TRUE) / 1024 / 1024, 2), 'Peak memory usage');
    if ($this->debugInfo['db_queries']) {
      $total_queries = count($this->databaseLogger->get('pc'));
      pc($this->databaseLogger->get('pc'), "DB queries ($total_queries)");
    }

  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    return [
      KernelEvents::REQUEST => ['onKernelRequest'],
      KernelEvents::RESPONSE => ['onKernelResponse'],
    ];
  }

}
